@@ -10,28 +10,36 @@ import decimal
 import os
 import json
 
-from PIL import Image  # require PIL module
+try:
+    from PIL import Image  # require PIL module
+except ImportError:
+    class _FakeImage(object):
+        """
+        Заглушка для PIL.Image
+        """
+        def __getattr__(self, attr):
+            raise ImportError("PIL not installed!")
+    Image = _FakeImage
 
 from django.core.files.base import ContentFile
 from django.conf import settings
-from django.utils.log import logger
+try:
+    from django.utils.log import logger
+except ImportError:
+    from django.utils.log import getLogger
+    logger = getLogger('django')
 
 from m3_ext.ui import render_template
-from m3_ext.ui.fields.base import BaseExtField
-from m3_ext.ui.fields import (
-    ExtNumberField,
-    ExtStringField,
-    ExtDateField,
-    ExtCheckBox, ExtComboBox, ExtTimeField, ExtRadio,
-    ExtHiddenField,
-    ExtFileUploadField, ExtImageUploadField,
-    ExtDictSelectField, ExtMultiSelectField
-)
 from m3.actions.interfaces import ISelectablePack, IMultiSelectablePack
 from m3_ext.ui.base import ExtUIComponent, BaseExtComponent
-from m3_ext.ui.fields.simple import ExtDateTimeField, ExtAdvTimeField
 
-from base import BaseExtPanel
+from m3_ext.ui.containers.base import BaseExtPanel
+
+
+
+def _is_field(obj):
+    from m3_ext.ui.fields.base import BaseExtField
+    return isinstance(obj, BaseExtField)
 
 
 class ExtForm(BaseExtPanel):
@@ -66,7 +74,7 @@ class ExtForm(BaseExtPanel):
         """
         if lst is None:
             lst = []
-        if isinstance(item, BaseExtField):
+        if _is_field(item):
             lst.append(item)
 
         elif hasattr(item, 'items'):
@@ -81,24 +89,33 @@ class ExtForm(BaseExtPanel):
         """
         assert request, 'Request must be define!'
 
+        from m3_ext.ui import fields
+
         all_fields = self._get_all_fields(self)
         for field in all_fields:
             name = field.name
-            if isinstance(field, (ExtFileUploadField, ExtImageUploadField)):
+            if isinstance(field, (
+                fields.ExtFileUploadField,
+                fields.ExtImageUploadField
+            )):
                 # Файлы нужно забирать из request.FILES
                 field.memory_file = request.FILES.get(
-                    ExtFileUploadField.PREFIX + field.name)
+                    fields.ExtFileUploadField.PREFIX + field.name)
             # возьмем только то, что есть в запросе
             if name in request.POST:
                 value = request.POST.get(name)
                 field.value = value
 
     #TODO необходимо добавить проверку на возникновение exception'ов
-    def from_object(self, object, exclusion=[]):
+    def from_object(self, obj, exclusion=None):
         """
-        Метод выполнения прямого связывания данных атрибутов объекта object и
+        Метод выполнения прямого связывания данных атрибутов объекта obj и
         полей текущей формы
         """
+        from m3_ext.ui import fields
+        from m3_ext.ui.fields import simple
+
+        exclusion = exclusion or []
 
         def is_secret_token(value):
             """
@@ -112,18 +129,18 @@ class ExtForm(BaseExtPanel):
             """
             Конвертирует и присваивает значение value в соответствии типу item.
             """
-            if isinstance(item, ExtStringField):
+            if isinstance(item, fields.ExtStringField):
                 if value:
                     item.value = unicode(value)
                 else:
                     item.value = u''
 
-            elif isinstance(item, ExtAdvTimeField):
+            elif isinstance(item, simple.ExtAdvTimeField):
                 item.value = '%02d:%02d:%02d' % (
                     value.hour, value.minute, value.second
                 ) if not is_secret_token(value) else unicode(value)
 
-            elif isinstance(item, ExtDateTimeField):
+            elif isinstance(item, simple.ExtDateTimeField):
                 if isinstance(value, datetime.datetime):
                     item.value = '%02d.%02d.%04d %02d:%02d:%02d' % (
                         value.day, value.month, value.year, value.hour,
@@ -136,7 +153,7 @@ class ExtForm(BaseExtPanel):
                 else:
                     item.value = value
 
-            elif isinstance(item, ExtDateField):
+            elif isinstance(item, fields.ExtDateField):
                 if isinstance(value, (datetime.date, datetime.datetime)):
                     item.value = '%02d.%02d.%04d' % (
                         value.day, value.month, value.year
@@ -144,7 +161,7 @@ class ExtForm(BaseExtPanel):
                 else:
                     item.value = value
 
-            elif isinstance(item, ExtTimeField):
+            elif isinstance(item, fields.ExtTimeField):
                 if isinstance(value, (datetime.time, datetime.datetime)):
                     item.value = '%02d:%02d' % (
                         value.hour, value.minute
@@ -152,11 +169,11 @@ class ExtForm(BaseExtPanel):
                 else:
                     item.value = value
 
-            elif isinstance(item, ExtCheckBox):
+            elif isinstance(item, fields.ExtCheckBox):
                 item.checked = True if value else False
-            elif isinstance(item, ExtRadio):
+            elif isinstance(item, fields.ExtRadio):
                 item.checked = (value == item.value)
-            elif isinstance(item, ExtMultiSelectField):
+            elif isinstance(item, fields.ExtMultiSelectField):
                 # У поля выбора может быть сзязанный с ним пак
                 bind_pack = getattr(item, 'pack', None) or getattr(
                     item, 'bind_pack', None)
@@ -170,7 +187,7 @@ class ExtForm(BaseExtPanel):
                         value_field=item.value_field,
                         display_field=item.display_field
                     )
-            elif isinstance(item, ExtDictSelectField):
+            elif isinstance(item, fields.ExtDictSelectField):
                 # У поля выбора может быть сзязанный с ним пак
                 # TODO после окончательного удаления метода
                 # configure_by_dictpack в ExtDictSelectField
@@ -193,7 +210,7 @@ class ExtForm(BaseExtPanel):
                         item.default_text = bind_pack.get_display_text(
                             value, item.display_field)
                 item.value = value
-            elif isinstance(item, ExtComboBox) and hasattr(
+            elif isinstance(item, fields.ExtComboBox) and hasattr(
                     item, 'bind_rule_reverse'):
                 # Комбобокс как правило передает id выбранного значения.
                 # Его не так просто  преобразовать в тип объекта,
@@ -210,7 +227,10 @@ class ExtForm(BaseExtPanel):
                         'Must be a function or a dict.'
                     )
 
-            elif isinstance(item, (ExtFileUploadField, ExtImageUploadField)):
+            elif isinstance(item, (
+                fields.ExtFileUploadField,
+                fields.ExtImageUploadField
+            )):
                 item.value = unicode(value)
                 # Относительную URL ссылку до статики
 
@@ -221,7 +241,7 @@ class ExtForm(BaseExtPanel):
                     item.file_url = None
 
                 # Прибиндим оригинальные размеры thumbnail
-                if isinstance(item, ExtImageUploadField):
+                if isinstance(item, fields.ExtImageUploadField):
                     if hasattr(settings, 'MEDIA_ROOT') and item.thumbnail:
                         ffile = os.path.join(
                             settings.MEDIA_ROOT, unicode(value))
@@ -230,7 +250,7 @@ class ExtForm(BaseExtPanel):
 
                         thumb_file = os.path.join(
                             dir_,
-                            (ExtImageUploadField.MIN_THUMBNAIL_PREFIX +
+                            (fields.ExtImageUploadField.MIN_THUMBNAIL_PREFIX +
                                 file_name))
                         if os.path.exists(thumb_file):
                             thumb = Image.open(thumb_file)
@@ -246,6 +266,10 @@ class ExtForm(BaseExtPanel):
 
             names задается в виде списка, т.о. если его длина больше единицы,
             то имеются вложенные объекты и их надо обработать
+            :param obj: объект или словарь
+            :type obj: object или dict
+            :param names: список имен
+            :type names: list
             """
 
             # hasattr не работает для dict'a
@@ -279,18 +303,34 @@ class ExtForm(BaseExtPanel):
             if not field.name in exclusion:
 
                 names = field.name.split('.')
-                new_val, has_attr = get_value(object, names)
+                new_val, has_attr = get_value(obj, names)
                 if has_attr:
                     new_val = new_val if new_val is not None else ''
                     _assign_value(new_val, field)
 
     #TODO необходимо добавить проверку на возникновение exception'ов
-    def to_object(self, object, exclusion=[]):
+    def to_object(self, obj, exclusion=None):
         '''
         Метод выполнения обратного связывания данных.
+        :param obj: объект
+        :type obj: object
+        :param exclusion: список исключаемых аттрибутов
+        :type exclusion: list
         '''
-        def _save_image(obj, name, field):
-            # Работа с изображением или файлом
+        from m3_ext.ui import fields
+        from m3_ext.ui.fields import simple
+
+        exclusion = exclusion or []
+
+        def save_image(obj, name, field):
+            """
+            Работа с изображением или файлом
+            :param field: поле
+            :type field: потомок m3_ext.ui.fields.complex.BaseExtField
+            :param obj: объект
+            :param name: имя поля
+            :type name: str
+            """
             if hasattr(obj, name):
                 l_field = getattr(obj, name)
                 if (
@@ -301,15 +341,15 @@ class ExtForm(BaseExtPanel):
                     ).lower() != (field.value).lower()
                 ):
                     # Сначало нужно удалить thumbnail картинки
-                    if isinstance(field, ExtImageUploadField) and (
+                    if isinstance(field, fields.ExtImageUploadField) and (
                             field.thumbnail):
                         current_dir = os.path.dirname(l_field.path)
                         basename = os.path.basename(l_field.path)
 
                         thumb_prefix = (
-                            ExtImageUploadField.MIDDLE_THUMBNAIL_PREFIX,
-                            ExtImageUploadField.MIN_THUMBNAIL_PREFIX,
-                            ExtImageUploadField.MAX_THUMBNAIL_PREFIX
+                            fields.ExtImageUploadField.MIDDLE_THUMBNAIL_PREFIX,
+                            fields.ExtImageUploadField.MIN_THUMBNAIL_PREFIX,
+                            fields.ExtImageUploadField.MAX_THUMBNAIL_PREFIX
                         )
 
                         for prefix in thumb_prefix:
@@ -329,7 +369,7 @@ class ExtForm(BaseExtPanel):
                     l_field = getattr(obj, name)
                     l_field.save(name_file, cont_file, save=False)
 
-                    if isinstance(field, ExtImageUploadField):
+                    if isinstance(field, fields.ExtImageUploadField):
                         try:
                             img = Image.open(l_field.path)
                         except IOError:
@@ -349,30 +389,30 @@ class ExtForm(BaseExtPanel):
                             #     (curr_width, curr_height), Image.ANTIALIAS)
                             img.save(l_field.path)
 
-                        if isinstance(field, ExtImageUploadField) and (
+                        if isinstance(field, fields.ExtImageUploadField) and (
                                 field.thumbnail and field.memory_file):
                             current_dir = os.path.dirname(l_field.path)
 
                             # А так же нужно сохранять thumbnail картинки
                             # Состовляем лист thumbnail_size'ов
-                            thumbnails = [
-                                (field.min_thumbnail_size,
-                                    ExtImageUploadField.MIN_THUMBNAIL_PREFIX)]
+                            thumbnails = [(
+                                field.min_thumbnail_size,
+                                fields.ExtImageUploadField.MIN_THUMBNAIL_PREFIX
+                            )]
 
                             if field.middle_thumbnail_size:
                                 thumbnails.append((
                                     field.middle_thumbnail_size,
-                                    ExtImageUploadField.MIDDLE_THUMBNAIL_PREFIX
+                                    fields.ExtImageUploadField.MIDDLE_THUMBNAIL_PREFIX
                                 ))
 
                             if field.max_thumbnail_size:
                                 thumbnails.append((
                                     field.max_thumbnail_size,
-                                    ExtImageUploadField.MAX_THUMBNAIL_PREFIX
+                                    fields.ExtImageUploadField.MAX_THUMBNAIL_PREFIX
                                 ))
 
-                            for size, prefix in thumbnails:
-
+                            for thumb in thumbnails:
                                 # Генерируем thumbnails
                                 # thumb_zise = get_img_size(size, img.size)
                                 # new_img = img.resize(
@@ -380,7 +420,7 @@ class ExtForm(BaseExtPanel):
 
                                 base_name = os.path.basename(l_field.path)
                                 tmb_path = os.path.join(
-                                    current_dir, prefix + base_name)
+                                    current_dir, thumb[1] + base_name)
                                 img.save(tmb_path)
 
         def set_field(obj, names, value, field=None):
@@ -391,6 +431,11 @@ class ExtForm(BaseExtPanel):
 
             names задается в виде списка, т.о. если его длина больше единицы,
             то имеются вложенные объекты
+            :param obj: объект
+            :param names: список имен
+            :param value: значение
+            :param field: поле
+            :type field: потомок m3_ext.ui.fields.complex.BaseExtField
             """
 
             # hasattr не работает для dict'a
@@ -401,9 +446,11 @@ class ExtForm(BaseExtPanel):
                 if len(names) == 1:
                     if isinstance(obj, dict):
                         obj[names[0]] = value
-                    elif isinstance(
-                            field, (ExtFileUploadField, ExtImageUploadField)):
-                        _save_image(obj, names[0], field)
+                    elif isinstance(field, (
+                        fields.ExtFileUploadField,
+                        fields.ExtImageUploadField
+                    )):
+                        save_image(obj, names[0], field)
 
                     else:
                         # Для id нельзя присваивать пустое значение!
@@ -422,6 +469,9 @@ class ExtForm(BaseExtPanel):
             """
             Пробует преобразовать value в целое число,
             иначе возвращает default
+            :param value: значение
+            :param default: значение, возвращаемое, если value не указано
+            :rtype: int
             """
             if not value:
                 return default
@@ -434,6 +484,8 @@ class ExtForm(BaseExtPanel):
         def try_to_list(value):
             """
             Пробует преобразовать value в список
+            :param value: значение
+            :rtype: list
             """
             if not value:
                 return []
@@ -447,9 +499,10 @@ class ExtForm(BaseExtPanel):
             """
             Берет значение item.value,
             и конвертирует его в соответствии с типом item'a
+            :type item: потомок m3_ext.ui.fields.complex.BaseExtField
             """
             val = item.value
-            if isinstance(item, ExtNumberField):
+            if isinstance(item, fields.ExtNumberField):
                 if val:
                     # для языков, где decimal разделяются не точкой
                     if item.decimal_separator:
@@ -463,35 +516,35 @@ class ExtForm(BaseExtPanel):
                             val = None
                 else:
                     val = None
-            elif isinstance(item, ExtStringField):
+            elif isinstance(item, fields.ExtStringField):
                 val = unicode(val) if val is not None else None
-            elif isinstance(item, ExtAdvTimeField):
+            elif isinstance(item, simple.ExtAdvTimeField):
                 if val and val.strip():
                     d = datetime.datetime.strptime(val, "%H:%M:%S")
                     val = d.time()
                 else:
                     val = None
-            elif isinstance(item, ExtDateTimeField):
+            elif isinstance(item, simple.ExtDateTimeField):
                 if val and val.strip():
                     val = datetime.datetime.strptime(val, '%d.%m.%Y %H:%M:%S')
                 else:
                     val = None
-            elif isinstance(item, ExtDateField):
+            elif isinstance(item, fields.ExtDateField):
                 #TODO уточнить формат дат
                 if val and val.strip():
                     d = datetime.datetime.strptime(val, '%d.%m.%Y')
                     val = d.date()
                 else:
                     val = None
-            elif isinstance(item, ExtTimeField):
+            elif isinstance(item, fields.ExtTimeField):
                 if val and val.strip():
                     d = datetime.datetime.strptime(val, '%H:%M')
                     val = datetime.time(d.hour, d.minute, 0)
                 else:
                     val = None
-            elif isinstance(item, ExtCheckBox):
+            elif isinstance(item, fields.ExtCheckBox):
                 val = True if val == 'on' else False
-            elif isinstance(item, ExtComboBox):
+            elif isinstance(item, fields.ExtComboBox):
                 # Комбобокс как правило передает id выбранного значения.
                 #Его не так просто преобразовать в тип объекта,
                 # т.к. мы ничего не знаем о структуре объекта.
@@ -511,16 +564,16 @@ class ExtForm(BaseExtPanel):
                 else:
                     val = try_to_int(val)
 
-            elif isinstance(item, ExtMultiSelectField):
+            elif isinstance(item, fields.ExtMultiSelectField):
                 val = try_to_list(val)
 
-            elif isinstance(item, ExtDictSelectField):
+            elif isinstance(item, fields.ExtDictSelectField):
                 val = try_to_int(val)
 
-            elif isinstance(item, ExtHiddenField):
-                if item.type == ExtHiddenField.INT:
+            elif isinstance(item, fields.ExtHiddenField):
+                if item.type == fields.ExtHiddenField.INT:
                     val = try_to_int(val)
-                elif item.type == ExtHiddenField.STRING:
+                elif item.type == fields.ExtHiddenField.STRING:
                     val = unicode(val)
             return val
 
@@ -528,7 +581,7 @@ class ExtForm(BaseExtPanel):
         # чтобы проверить возможность их сохранения
         try:
             list_of_m2m = [
-                x[0].name for x in object._meta.get_m2m_with_model()]
+                x[0].name for x in obj._meta.get_m2m_with_model()]
         except AttributeError:
             list_of_m2m = []
 
@@ -549,27 +602,28 @@ class ExtForm(BaseExtPanel):
             # заполним атрибуты только те, которые не в списке исключаемых
             if not field.name in exclusion:
                 # запрещаем пытаться сохранять many2many для объекта без pk
-                if hasattr(object, 'pk'):
-                    if object.pk is None and field.name in list_of_m2m:
+                if hasattr(obj, 'pk'):
+                    if obj.pk is None and field.name in list_of_m2m:
                         raise ValueError(' '.join(
                             ["'%s' instance needs to have a primary" % (
-                                object.__class__.__name__),
+                                obj.__class__.__name__),
                              "key value before a many-to-many "
                              "relationship can be used."]))
 
                 names = field.name.split('.')
-                set_field(object, names, convert_value(field), field)
+                set_field(obj, names, convert_value(field), field)
 
     @property
     def items(self):
         return self._items
 
     def pre_render(self):
+        from m3_ext.ui.fields import ExtHiddenField
         super(ExtForm, self).pre_render()
         if not self.focused_field:
             childs = self._get_all_fields(self)
             for child in childs:
-                if isinstance(child, BaseExtField) and not isinstance(
+                if _is_field(child) and not isinstance(
                         child, ExtHiddenField) and not child.hidden:
                     self.focused_field = child
                     break
@@ -642,7 +696,6 @@ class ExtPanel(BaseExtPanel):
         """
         Рендерит и возвращает js-код, который помещен в template_globals
         """
-        self.pre_render_globals()
         if self.template_globals:
             return render_template(
                 self.template_globals,
